<?php

App::uses('AppComponent', 'Controller/Component');

/**
 * Discount code component - listens in all post data to see if a discount code
 * is being applied.
 */
class DiscountCodesComponent extends AppComponent {

	/**
	 * Component startup - check for the form being submitted
	 *
	 * @param Controller $Controller Controller instance
	 * @return bool Returns true if discount applied. A flash status message is also set
	 */
	public function startup(Controller $Controller) {
		if ($this->_controller->request->is('post') && ! empty($this->_controller->request->data['Discount'])) {
			$this->_setupCookie();
			$discountCookie = $this->_controller->DiscountCookie->read('Code');

			if (! empty($discountCookie) && Configure::read('EvDiscount.multipleCodes') !== true) {
				$this->_controller->Flash->fail(
					__('A discount code has already been applied'),
					array(
						'key' => 'discount-flash'
					)
				);

				return false;
			}

			$code = $this->_controller->request->data['Discount']['DiscountCode']['code'];
			if (! empty($code)) {

				// Clean up the discount code as it's going to be pretty common
				// for people to copy/paste and accidentaly get a space round it.
				$code = trim($code);

				$DiscountCode = EvClassRegistry::init('EvDiscount.DiscountCode');
				$data = $DiscountCode->getCode($code);

				if (empty($data)) {
					$this->_controller->Flash->fail(
						__('The code entered is not valid'),
						array(
							'key' => 'discount-flash'
						)
					);

					return false;
				}

				// encrypt the code ready to be written to its cookie
				$encryptedData = $DiscountCode->hashForCookie($data);

				$this->_controller->DiscountCookie->write('Code', $encryptedData);

				// dispatch event
				$this->_controller->getEventManager()->dispatch(
					new CakeEvent('EvDiscount.Component.DiscountCodes.codeAdded', $this, array(
						'code' => $code,
						'data' => $data
					))
				);

				$this->_controller->Flash->success(
					__('Discount code applied'),
					array(
						'key' => 'discount-flash'
					)
				);
			}
		}

		return true;
	}

	/**
	 * setup the discount cookie
	 *
	 * @return void
	 */
	protected function _setupCookie() {
		if (empty($this->_controller->DiscountCookie)) {
			$this->_controller->DiscountCookie = $this->_controller->loadComponent('EvDiscount.DiscountCookie');
		}
	}

}